<?php
// $Id: gmap_blocks.content.inc,v 1.1 2009/07/06 07:23:41 skilip Exp $

/**
 * @file
 *
 * Include file for declaring functionality needed for an GMap Blocks block
 */

/**
 * Content callback for gmap_blocks_block().
 * Generates an array for the block implemantation, adds required css and js files
 * and prepares the javascript settings
 *
 * @param delta (number)
 *   - The block's delta which equals the block's database bid
 *
 * @see gmap_blocks_block()
 */
function gmap_blocks_content($delta) {
  $path = drupal_get_path('module', 'gmap_blocks');
  $block = gmap_blocks_block_load($delta);

  $width = $block['settings']['width'];
  $height = $block['settings']['height'];
  $name = $block['name'];

  // Unset unneeded elements
  unset($block['settings']['width'], $block['settings']['height'], $block['name']);

  // Merge the address
  foreach ($block['locations'] as $lid => $location) {
    $address = array();

    foreach (array('address', 'city', 'state', 'country') as $key) {
      $address[] = $block['locations'][$lid][$key];

      // In javascript we will only use the full address string, so we can unset the rest here
      unset($block['locations'][$lid][$key]);
    }
    $block['locations'][$lid]['address'] = join(', ', $address);
    $block['locations'][$lid]['marker'] = (bool) $location['marker'];

    if ($location['directions']) {
      $block['locations'][$lid]['info'] .= drupal_get_form('gmap_blocks_directions_form', $delta, $location);
      drupal_add_css("$path/gmap_blocks.css");
    }

    if (!empty($block['locations'][$lid]['info'])) {
      $block['locations'][$lid]['info'] = "<div id=\"gmap-blocks-info-window-$delta-$lid\">". $block['locations'][$lid]['info'] ."</div>\n";
    }
  }

  // Store the id of the div element the map has to placed in by javascript
  $block['target_id'] = 'gmap-blocks-'. $delta;

  drupal_add_js(array('gmap_blocks' => array($delta => $block)), 'setting');
  drupal_set_html_head('<script type="text/javascript" src="http://maps.google.com/maps/api/js?sensor=false"></script>');
  drupal_add_js("$path/gmap_blocks.js");

  return array(
    'subject' => $name,
    'content' => '<div id="'. $block['target_id'] .'" style="width:'. $width .'px; height:'. $height .'px"></div>',
  );
}

/**
 * Declaration of a little form used inside info windows for getting directions
 * This form will only be displayed if the user has checked the 'directions' checkbox for the location element
 */
function gmap_blocks_directions_form($form_sate, $delta, $location) {
  $form = array();

  $form['visible'] = array(
    '#prefix' => '<div class="gmap-blocks-directions-visible">',
    '#value' => '<a href="javascript:Drupal.GMapBlocks['. $delta .'].initDirectionsForm()">'. t('Plan route...') .'</a>',
    '#suffix' => '</div>',
  );

  $form['hidden'] = array(
    '#prefix' => '<div class="gmap-blocks-directions-hidden">',
    '#suffix' => '</div>',
  );
  $form['hidden']['source_address'] = array(
    '#type' => 'textfield',
    '#default_value' => t('Traveling from'),
    '#size' => 20,
    '#maxlength' => 128,
  );
  $form['hidden']['submit'] = array('#type' => 'submit', '#value' => t('Directions'));

  $form['direction_address'] = array('#type' => 'hidden', '#default_value' => '');
  $form['lid'] = array('#type' => 'hidden', '#default_value' => $location['lid']);
  $form['bid'] = array('#type' => 'hidden', '#default_value' => $delta);

  return $form;
}